// ------------------------------------------------------------------------------------
// <copyright file="PaisesBL.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.Negocio.Clases.BL
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Biblioteca.Datos.Clases.DAL.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.IC.DTO.Repositorio;
	using Biblioteca.IC.Recursos;
	using TransaccionalNet.EF.Clases;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using Utilitarios.TransversalesNet.Clases.Comunes.Enumeraciones;

	/// <summary>
	/// Clase con las acciones de negocio de la entidad Paises
	/// </summary>
	public class PaisesBL : AccesoComunBL, IPaisesNegocioAccion
	{

		#region ATRIBUTOS
		/// <summary>
		/// Objeto para repositorio Paises
		/// <summary>
		/// Objeto para acciones Paises
		/// </summary>
		private Lazy<IPaisesRepositorioAccion> paisesRepositorioAccion;

		/// <summary>
		/// Objeto para entidad respuesta
		/// </summary>
		Respuesta<IPaisesDTO> respuesta;

		#endregion ATRIBUTOS
		#region CONSTRUCTORES
		/// <summary>
		/// Inicializa una nueva instancia de la clase PaisesBL
		/// </summary>
		/// <param name="argPaisesRepositorioAccion">Acciones entidad Paises</param>
		public PaisesBL(Lazy<IPaisesRepositorioAccion> argPaisesRepositorioAccion = null)
		{
			this.respuesta = new Respuesta<IPaisesDTO>();
			this.paisesRepositorioAccion = argPaisesRepositorioAccion ?? new Lazy<IPaisesRepositorioAccion>(() => new PaisesDAL());
		}

		#endregion CONSTRUCTORES
		#region METODOS PUBLICOS
		/// <summary>
		/// Metodo consultar lista paises
		/// </summary>
		/// <returns>Respuesta tipo Paises </returns>
		public async Task<Respuesta<IPaisesDTO>> ConsultarListaPaisesAsync()
		{

			return await this.EjecutarTransaccionBDAsync<Respuesta<IPaisesDTO>, PaisesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.paisesRepositorioAccion.Value.ConsultarListaPaisesAsync();
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});

		}
		/// <summary>
		/// Metodo consultar por llave paises
		/// </summary>
		/// <param name="paises">Entidad a consultar</param>
		/// <returns>Respuesta tipo Paises </returns>
		public async Task<Respuesta<IPaisesDTO>> ConsultarPaisesLlaveAsync(IPaisesDTO paises)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IPaisesDTO>, PaisesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.paisesRepositorioAccion.Value.ConsultarPaisesLlaveAsync(paises);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo editar paises
		/// </summary>
		/// <param name="paises">Entidad a editar</param>
		/// <returns>Respuesta tipo Paises </returns>
		public async Task<Respuesta<IPaisesDTO>> EditarPaisesAsync(IPaisesDTO paises)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IPaisesDTO>, PaisesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.paisesRepositorioAccion.Value.EditarPaisesAsync(paises);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}


		/// <summary>
		/// Metodo eliminar paises
		/// </summary>
		/// <param name="paises">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Paises </returns>
		public async Task<Respuesta<IPaisesDTO>> EliminarPaisesAsync(IPaisesDTO paises)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IPaisesDTO>, PaisesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.paisesRepositorioAccion.Value.EliminarPaisesAsync(paises);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeEntidadEliminadaConExito);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo guardar paises
		/// </summary>
		/// <param name="paises">Entidad a guardar</param>
		/// <returns>Respuesta tipo Paises </returns>
		public async Task<Respuesta<IPaisesDTO>> GuardarPaisesAsync(IPaisesDTO paises)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IPaisesDTO>, PaisesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.paisesRepositorioAccion.Value.GuardarPaisesAsync(paises);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo Guardar lista paises
		/// </summary>
		/// <param name="listaPaises">Lista de entidades a guardar</param>
		/// <returns>Respuesta tipo Paises </returns>
		public async Task<Respuesta<IPaisesDTO>> GuardarListaPaisesAsync(List<IPaisesDTO> listaPaises)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IPaisesDTO>, PaisesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.paisesRepositorioAccion.Value.GuardarListaPaisesAsync(listaPaises);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}
		#endregion METODOS PUBLICOS
		#region METODOS PRIVADOS
		#endregion METODOS PRIVADOS
	}
}

