// ------------------------------------------------------------------------------------
// <copyright file="PaisesController.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.API.Controllers
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Utilitarios.APINet.Clases.Ejecucion;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using UtilitariosNet.Clases.Mapeador.Extensiones;
	using Biblioteca.API.Models.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.Negocio.Clases.BL;
	using Microsoft.AspNetCore.Mvc;
	
	/// <summary>
	/// Clase con las capaciades Rest de la entidad Paises
	/// </summary>
	[Route("api/Paises")]
	public class PaisesController : AccesoComunAPI
	{


		/// <summary>
		/// Objeto para negocio Paises
		/// </summary>
		private Lazy<IPaisesNegocioAccion> negocioPaises;

		
		/// <summary>
		/// Inicializa una nueva instancia de la clase PaisesController
		/// </summary>
		public PaisesController()
		{
			this.negocioPaises = new Lazy<IPaisesNegocioAccion>(
										() => new PaisesBL());
		}
		
		/// <summary>
		/// Metodo consultar lista paises
		/// </summary>
		/// <returns>Respuesta tipo Paises </returns>
		[HttpGet]
		[Route("ConsultarListaPaises")]
		public async Task<Respuesta<Paises>> ConsultarListaPaises()
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Paises>>, PaisesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Paises>>(await negocioPaises.Value.ConsultarListaPaisesAsync());
			});
		}
		
		/// <summary>
		/// Metodo consultar por llave paises
		/// </summary>
		/// <param name="paises">Entidad a consultar</param>
		/// <returns>Respuesta tipo Paises </returns>
		[HttpPost]
		[Route("ConsultarPaisesLlave")]
		public async Task<Respuesta<Paises>> ConsultarPaisesLlave(Paises paises)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Paises>>, PaisesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Paises>>(await negocioPaises.Value.ConsultarPaisesLlaveAsync(paises));
			});
		}
		
		/// <summary>
		/// Metodo guardar paises
		/// </summary>
		/// <param name="paises">Entidad a guardar</param>
		/// <returns>Respuesta tipo Paises </returns>
		[HttpPost]
		[Route("GuardarPaises")]
		public async Task<Respuesta<Paises>> GuardarPaises(Paises paises)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Paises>>, PaisesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Paises>>(await negocioPaises.Value.GuardarPaisesAsync(paises));
			});
		}
		
		/// <summary>
		/// Metodo editar paises
		/// </summary>
		/// <param name="paises">Entidad a editar</param>
		/// <returns>Respuesta tipo Paises </returns>
		[HttpPut]
		[Route("EditarPaises")]
		public async Task<Respuesta<Paises>> EditarPaises(Paises paises)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Paises>>, PaisesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Paises>>(await negocioPaises.Value.EditarPaisesAsync(paises));
			});
		}
		
		/// <summary>
		/// Metodo eliminar paises
		/// </summary>
		/// <param name="paises">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Paises </returns>
		[HttpDelete]
		[Route("EliminarPaises")]
		public async Task<Respuesta<Paises>> EliminarPaises(Paises paises)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Paises>>, PaisesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Paises>>(await negocioPaises.Value.EliminarPaisesAsync(paises));
			});
		}
	}
}

